from jdaviz.core.tools import SidebarShortcutPlotOptions, SidebarShortcutExportPlot

# point to the lcviz-version of plot options instead of jdaviz's
SidebarShortcutPlotOptions.plugin_name = 'lcviz-plot-options'
SidebarShortcutExportPlot.plugin_name = 'lcviz-export-plot'
